@@ -17,11 +17,9 @@
 
 use arrow::datatypes::Schema;
 use arrow::error::{ArrowError, Result as ArrowResult};
-use arrow::ipc::{
-    convert, size_prefixed_root_as_message, writer, writer::EncodedData,
-    writer::IpcWriteOptions,
-};
+use arrow::ipc::{convert, writer, writer::EncodedData, writer::IpcWriteOptions};
 
+use arrow::ipc::convert::try_schema_from_ipc_buffer;
 use std::{
     convert::{TryFrom, TryInto},
     fmt,
@@ -254,10 +252,17 @@ impl From<SchemaAsIpc<'_>> for FlightData {
     }
 }
 
-impl From<SchemaAsIpc<'_>> for SchemaResult {
-    fn from(schema_ipc: SchemaAsIpc) -> Self {
-        let IpcMessage(vals) = flight_schema_as_flatbuffer(schema_ipc.0, schema_ipc.1);
-        SchemaResult { schema: vals }
+impl TryFrom<SchemaAsIpc<'_>> for SchemaResult {
+    type Error = ArrowError;
+
+    fn try_from(schema_ipc: SchemaAsIpc) -> ArrowResult<Self> {
+        // According to the definition from `Flight.proto`
+        // The schema of the dataset in its IPC form:
+        //   4 bytes - an optional IPC_CONTINUATION_TOKEN prefix
+        //   4 bytes - the byte length of the payload
+        //   a flatbuffer Message whose header is the Schema
+        let IpcMessage(vals) = schema_to_ipc_format(schema_ipc)?;
+        Ok(SchemaResult { schema: vals })
     }
 }
 
@@ -275,19 +280,23 @@ impl TryFrom<SchemaAsIpc<'_>> for IpcMessage {
     type Error = ArrowError;
 
     fn try_from(schema_ipc: SchemaAsIpc) -> ArrowResult<Self> {
-        let pair = *schema_ipc;
-        let encoded_data = flight_schema_as_encoded_data(pair.0, pair.1);
-
-        let mut schema = vec![];
-        arrow::ipc::writer::write_message(&mut schema, encoded_data, pair.1)?;
-        Ok(IpcMessage(schema))
+        schema_to_ipc_format(schema_ipc)
     }
 }
 
+fn schema_to_ipc_format(schema_ipc: SchemaAsIpc) -> ArrowResult<IpcMessage> {
+    let pair = *schema_ipc;
+    let encoded_data = flight_schema_as_encoded_data(pair.0, pair.1);
+
+    let mut schema = vec![];
+    arrow::ipc::writer::write_message(&mut schema, encoded_data, pair.1)?;
+    Ok(IpcMessage(schema))
+}
+
 impl TryFrom<&FlightData> for Schema {
     type Error = ArrowError;
     fn try_from(data: &FlightData) -> ArrowResult<Self> {
-        convert::schema_from_bytes(&data.data_header[..]).map_err(|err| {
+        convert::try_schema_from_flatbuffer_bytes(&data.data_header[..]).map_err(|err| {
             ArrowError::ParseError(format!(
                 "Unable to convert flight data to Arrow schema: {}",
                 err
@@ -309,32 +318,14 @@ impl TryFrom<IpcMessage> for Schema {
     type Error = ArrowError;
 
     fn try_from(value: IpcMessage) -> ArrowResult<Self> {
-        // CONTINUATION TAKES 4 BYTES
-        // SIZE TAKES 4 BYTES (so read msg as size prefixed)
-        let msg = size_prefixed_root_as_message(&value.0[4..]).map_err(|err| {
-            ArrowError::ParseError(format!(
-                "Unable to convert flight info to a message: {}",
-                err
-            ))
-        })?;
-        let ipc_schema = msg.header_as_schema().ok_or_else(|| {
-            ArrowError::ParseError(
-                "Unable to convert flight info to a schema".to_string(),
-            )
-        })?;
-        Ok(convert::fb_to_schema(ipc_schema))
+        try_schema_from_ipc_buffer(value.0.as_slice())
     }
 }
 
 impl TryFrom<&SchemaResult> for Schema {
     type Error = ArrowError;
     fn try_from(data: &SchemaResult) -> ArrowResult<Self> {
-        convert::schema_from_bytes(&data.schema[..]).map_err(|err| {
-            ArrowError::ParseError(format!(
-                "Unable to convert schema result to Arrow schema: {}",
-                err
-            ))
-        })
+        try_schema_from_ipc_buffer(data.schema.as_slice())
     }
 }
 
@@ -405,6 +396,8 @@ impl<'a> SchemaAsIpc<'a> {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use arrow::datatypes::{DataType, Field, TimeUnit};
+    use arrow::ipc::MetadataVersion;
 
     struct TestVector(Vec<u8>, usize);
 
@@ -448,4 +441,31 @@ mod tests {
         let expected = format!("{:?}", vec![91; 9]);
         assert_eq!(actual, expected);
     }
+
+    #[test]
+    fn ser_deser_schema_result() {
+        let schema = Schema::new(vec![
+            Field::new("c1", DataType::Utf8, false),
+            Field::new("c2", DataType::Float64, true),
+            Field::new("c3", DataType::UInt32, false),
+            Field::new("c4", DataType::Boolean, true),
+            Field::new("c5", DataType::Timestamp(TimeUnit::Millisecond, None), true),
+            Field::new("c6", DataType::Time32(TimeUnit::Second), false),
+        ]);
+        // V5 with write_legacy_ipc_format = false
+        // this will write the continuation marker
+        let option = IpcWriteOptions::default();
+        let schema_ipc = SchemaAsIpc::new(&schema, &option);
+        let result: SchemaResult = schema_ipc.try_into().unwrap();
+        let des_schema: Schema = (&result).try_into().unwrap();
+        assert_eq!(schema, des_schema);
+
+        // V4 with write_legacy_ipc_format = true
+        // this will not write the continuation marker
+        let option = IpcWriteOptions::try_new(8, true, MetadataVersion::V4).unwrap();
+        let schema_ipc = SchemaAsIpc::new(&schema, &option);
+        let result: SchemaResult = schema_ipc.try_into().unwrap();
+        let des_schema: Schema = (&result).try_into().unwrap();
+        assert_eq!(schema, des_schema);
+    }
 }
