@@ -43,7 +43,7 @@ fn bench_primitive(c: &mut Criterion) {
     ));
     group.bench_function("bench_primitive", |b| {
         b.iter(|| {
-            let mut builder = Int64Builder::new(64);
+            let mut builder = Int64Builder::with_capacity(64);
             for _ in 0..NUM_BATCHES {
                 builder.append_slice(&data[..]);
             }
@@ -57,7 +57,7 @@ fn bench_primitive_nulls(c: &mut Criterion) {
     let mut group = c.benchmark_group("bench_primitive_nulls");
     group.bench_function("bench_primitive_nulls", |b| {
         b.iter(|| {
-            let mut builder = UInt8Builder::new(64);
+            let mut builder = UInt8Builder::with_capacity(64);
             for _ in 0..NUM_BATCHES * BATCH_SIZE {
                 builder.append_null();
             }
@@ -80,7 +80,7 @@ fn bench_bool(c: &mut Criterion) {
     ));
     group.bench_function("bench_bool", |b| {
         b.iter(|| {
-            let mut builder = BooleanBuilder::new(64);
+            let mut builder = BooleanBuilder::with_capacity(64);
             for _ in 0..NUM_BATCHES {
                 builder.append_slice(&data[..]);
             }
@@ -98,7 +98,7 @@ fn bench_string(c: &mut Criterion) {
     ));
     group.bench_function("bench_string", |b| {
         b.iter(|| {
-            let mut builder = StringBuilder::new(64);
+            let mut builder = StringBuilder::new();
             for _ in 0..NUM_BATCHES * BATCH_SIZE {
                 builder.append_value(SAMPLE_STRING);
             }
@@ -112,7 +112,7 @@ fn bench_decimal128(c: &mut Criterion) {
     c.bench_function("bench_decimal128_builder", |b| {
         b.iter(|| {
             let mut rng = rand::thread_rng();
-            let mut decimal_builder = Decimal128Builder::new(BATCH_SIZE, 38, 0);
+            let mut decimal_builder = Decimal128Builder::with_capacity(BATCH_SIZE, 38, 0);
             for _ in 0..BATCH_SIZE {
                 decimal_builder
                     .append_value(rng.gen_range::<i128, _>(0..9999999999))
@@ -127,7 +127,8 @@ fn bench_decimal256(c: &mut Criterion) {
     c.bench_function("bench_decimal128_builder", |b| {
         b.iter(|| {
             let mut rng = rand::thread_rng();
-            let mut decimal_builder = Decimal256Builder::new(BATCH_SIZE, 76, 10);
+            let mut decimal_builder =
+                Decimal256Builder::with_capacity(BATCH_SIZE, 76, 10);
             for _ in 0..BATCH_SIZE {
                 decimal_builder
                     .append_value(&Decimal256::from(BigInt::from(
