@@ -265,8 +265,8 @@ mod tests {
     use crate::array::{
         array::Array, ArrayData, ArrayDataBuilder, ArrayRef, BooleanArray,
         FixedSizeBinaryBuilder, FixedSizeListBuilder, GenericBinaryArray, Int32Builder,
-        ListBuilder, NullArray, PrimitiveBuilder, StringArray, StringDictionaryBuilder,
-        StructArray, UnionBuilder,
+        ListBuilder, NullArray, StringArray, StringDictionaryBuilder, StructArray,
+        UnionBuilder,
     };
     use crate::array::{GenericStringArray, Int32Array};
     use crate::buffer::Buffer;
@@ -1245,8 +1245,8 @@ mod tests {
 
     fn create_dictionary_array(values: &[&str], keys: &[Option<&str>]) -> ArrayData {
         let values = StringArray::from(values.to_vec());
-        let mut builder = StringDictionaryBuilder::new_with_dictionary(
-            PrimitiveBuilder::<Int16Type>::with_capacity(3),
+        let mut builder = StringDictionaryBuilder::<Int16Type>::new_with_dictionary(
+            keys.len(),
             &values,
         )
         .unwrap();
