@@ -22,8 +22,6 @@ use std::sync::Arc;
 
 use num::Zero;
 
-use TimeUnit::*;
-
 use crate::array::*;
 use crate::buffer::{buffer_bin_and, Buffer, MutableBuffer};
 use crate::datatypes::*;
@@ -31,6 +29,7 @@ use crate::error::{ArrowError, Result};
 use crate::record_batch::RecordBatch;
 use crate::util::bit_iterator::{BitIndexIterator, BitSliceIterator};
 use crate::util::bit_util;
+use crate::{downcast_dictionary_array, downcast_primitive_array};
 
 /// If the filter selects more than this fraction of rows, use
 /// [`SlicesIterator`] to copy ranges of values. Otherwise iterate
@@ -40,27 +39,6 @@ use crate::util::bit_util;
 ///
 const FILTER_SLICES_SELECTIVITY_THRESHOLD: f64 = 0.8;
 
-macro_rules! downcast_filter {
-    ($type: ty, $values: expr, $filter: expr) => {{
-        let values = $values
-            .as_any()
-            .downcast_ref::<PrimitiveArray<$type>>()
-            .expect("Unable to downcast to a primitive array");
-
-        Ok(Arc::new(filter_primitive::<$type>(&values, $filter)))
-    }};
-}
-
-macro_rules! downcast_dict_filter {
-    ($type: ty, $values: expr, $filter: expr) => {{
-        let values = $values
-            .as_any()
-            .downcast_ref::<DictionaryArray<$type>>()
-            .expect("Unable to downcast to a dictionary array");
-        Ok(Arc::new(filter_dict::<$type>(values, $filter)))
-    }};
-}
-
 /// An iterator of `(usize, usize)` each representing an interval
 /// `[start, end)` whose slots of a [BooleanArray] are true. Each
 /// interval corresponds to a contiguous region of memory to be
@@ -358,92 +336,12 @@ fn filter_array(values: &dyn Array, predicate: &FilterPredicate) -> Result<Array
         IterationStrategy::None => Ok(new_empty_array(values.data_type())),
         IterationStrategy::All => Ok(make_array(values.data().slice(0, predicate.count))),
         // actually filter
-        _ => match values.data_type() {
+        _ => downcast_primitive_array! {
+            values => Ok(Arc::new(filter_primitive(values, predicate))),
             DataType::Boolean => {
                 let values = values.as_any().downcast_ref::<BooleanArray>().unwrap();
                 Ok(Arc::new(filter_boolean(values, predicate)))
             }
-            DataType::Int8 => {
-                downcast_filter!(Int8Type, values, predicate)
-            }
-            DataType::Int16 => {
-                downcast_filter!(Int16Type, values, predicate)
-            }
-            DataType::Int32 => {
-                downcast_filter!(Int32Type, values, predicate)
-            }
-            DataType::Int64 => {
-                downcast_filter!(Int64Type, values, predicate)
-            }
-            DataType::UInt8 => {
-                downcast_filter!(UInt8Type, values, predicate)
-            }
-            DataType::UInt16 => {
-                downcast_filter!(UInt16Type, values, predicate)
-            }
-            DataType::UInt32 => {
-                downcast_filter!(UInt32Type, values, predicate)
-            }
-            DataType::UInt64 => {
-                downcast_filter!(UInt64Type, values, predicate)
-            }
-            DataType::Float32 => {
-                downcast_filter!(Float32Type, values, predicate)
-            }
-            DataType::Float64 => {
-                downcast_filter!(Float64Type, values, predicate)
-            }
-            DataType::Date32 => {
-                downcast_filter!(Date32Type, values, predicate)
-            }
-            DataType::Date64 => {
-                downcast_filter!(Date64Type, values, predicate)
-            }
-            DataType::Time32(Second) => {
-                downcast_filter!(Time32SecondType, values, predicate)
-            }
-            DataType::Time32(Millisecond) => {
-                downcast_filter!(Time32MillisecondType, values, predicate)
-            }
-            DataType::Time64(Microsecond) => {
-                downcast_filter!(Time64MicrosecondType, values, predicate)
-            }
-            DataType::Time64(Nanosecond) => {
-                downcast_filter!(Time64NanosecondType, values, predicate)
-            }
-            DataType::Timestamp(Second, _) => {
-                downcast_filter!(TimestampSecondType, values, predicate)
-            }
-            DataType::Timestamp(Millisecond, _) => {
-                downcast_filter!(TimestampMillisecondType, values, predicate)
-            }
-            DataType::Timestamp(Microsecond, _) => {
-                downcast_filter!(TimestampMicrosecondType, values, predicate)
-            }
-            DataType::Timestamp(Nanosecond, _) => {
-                downcast_filter!(TimestampNanosecondType, values, predicate)
-            }
-            DataType::Interval(IntervalUnit::YearMonth) => {
-                downcast_filter!(IntervalYearMonthType, values, predicate)
-            }
-            DataType::Interval(IntervalUnit::DayTime) => {
-                downcast_filter!(IntervalDayTimeType, values, predicate)
-            }
-            DataType::Interval(IntervalUnit::MonthDayNano) => {
-                downcast_filter!(IntervalMonthDayNanoType, values, predicate)
-            }
-            DataType::Duration(TimeUnit::Second) => {
-                downcast_filter!(DurationSecondType, values, predicate)
-            }
-            DataType::Duration(TimeUnit::Millisecond) => {
-                downcast_filter!(DurationMillisecondType, values, predicate)
-            }
-            DataType::Duration(TimeUnit::Microsecond) => {
-                downcast_filter!(DurationMicrosecondType, values, predicate)
-            }
-            DataType::Duration(TimeUnit::Nanosecond) => {
-                downcast_filter!(DurationNanosecondType, values, predicate)
-            }
             DataType::Utf8 => {
                 let values = values
                     .as_any()
@@ -458,19 +356,10 @@ fn filter_array(values: &dyn Array, predicate: &FilterPredicate) -> Result<Array
                     .unwrap();
                 Ok(Arc::new(filter_string::<i64>(values, predicate)))
             }
-            DataType::Dictionary(key_type, _) => match key_type.as_ref() {
-                DataType::Int8 => downcast_dict_filter!(Int8Type, values, predicate),
-                DataType::Int16 => downcast_dict_filter!(Int16Type, values, predicate),
-                DataType::Int32 => downcast_dict_filter!(Int32Type, values, predicate),
-                DataType::Int64 => downcast_dict_filter!(Int64Type, values, predicate),
-                DataType::UInt8 => downcast_dict_filter!(UInt8Type, values, predicate),
-                DataType::UInt16 => downcast_dict_filter!(UInt16Type, values, predicate),
-                DataType::UInt32 => downcast_dict_filter!(UInt32Type, values, predicate),
-                DataType::UInt64 => downcast_dict_filter!(UInt64Type, values, predicate),
-                t => {
-                    unimplemented!("Filter not supported for dictionary key type {:?}", t)
-                }
-            },
+            DataType::Dictionary(_, _) => downcast_dictionary_array! {
+                values => Ok(Arc::new(filter_dict(values, predicate))),
+                t => unimplemented!("Filter not supported for dictionary type {:?}", t)
+            }
             _ => {
                 // fallback to using MutableArrayData
                 let mut mutable = MutableArrayData::new(
@@ -1039,7 +928,7 @@ mod tests {
     #[test]
     fn test_filter_string_array_with_negated_boolean_array() {
         let a = StringArray::from(vec!["hello", " ", "world", "!"]);
-        let mut bb = BooleanBuilder::new(2);
+        let mut bb = BooleanBuilder::with_capacity(2);
         bb.append_value(false);
         bb.append_value(true);
         bb.append_value(false);
@@ -1416,7 +1305,7 @@ mod tests {
     #[test]
     fn test_filter_map() {
         let mut builder =
-            MapBuilder::new(None, StringBuilder::new(16), Int64Builder::new(4));
+            MapBuilder::new(None, StringBuilder::new(), Int64Builder::with_capacity(4));
         // [{"key1": 1}, {"key2": 2, "key3": 3}, null, {"key1": 1}
         builder.keys().append_value("key1");
         builder.values().append_value(1);
@@ -1438,7 +1327,7 @@ mod tests {
         let got = filter(&maparray, &indices).unwrap();
 
         let mut builder =
-            MapBuilder::new(None, StringBuilder::new(8), Int64Builder::new(2));
+            MapBuilder::new(None, StringBuilder::new(), Int64Builder::with_capacity(2));
         builder.keys().append_value("key1");
         builder.values().append_value(1);
         builder.append(true).unwrap();
@@ -1553,7 +1442,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_dense(1);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         let expected_array = builder.build().unwrap();
 
@@ -1563,7 +1452,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_dense(2);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Int32Type>("A", 34).unwrap();
         let expected_array = builder.build().unwrap();
@@ -1574,7 +1463,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_dense(2);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Float64Type>("B", 3.2).unwrap();
         let expected_array = builder.build().unwrap();
@@ -1584,7 +1473,7 @@ mod tests {
 
     #[test]
     fn test_filter_union_array_dense() {
-        let mut builder = UnionBuilder::new_dense(3);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Float64Type>("B", 3.2).unwrap();
         builder.append::<Int32Type>("A", 34).unwrap();
@@ -1595,7 +1484,7 @@ mod tests {
 
     #[test]
     fn test_filter_run_union_array_dense() {
-        let mut builder = UnionBuilder::new_dense(3);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Int32Type>("A", 3).unwrap();
         builder.append::<Int32Type>("A", 34).unwrap();
@@ -1605,7 +1494,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_dense(3);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Int32Type>("A", 3).unwrap();
         let expected = builder.build().unwrap();
@@ -1615,7 +1504,7 @@ mod tests {
 
     #[test]
     fn test_filter_union_array_dense_with_nulls() {
-        let mut builder = UnionBuilder::new_dense(4);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Float64Type>("B", 3.2).unwrap();
         builder.append_null::<Float64Type>("B").unwrap();
@@ -1626,7 +1515,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_dense(2);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Float64Type>("B", 3.2).unwrap();
         let expected_array = builder.build().unwrap();
@@ -1637,7 +1526,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_dense(2);
+        let mut builder = UnionBuilder::new_dense();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append_null::<Float64Type>("B").unwrap();
         let expected_array = builder.build().unwrap();
@@ -1647,7 +1536,7 @@ mod tests {
 
     #[test]
     fn test_filter_union_array_sparse() {
-        let mut builder = UnionBuilder::new_sparse(3);
+        let mut builder = UnionBuilder::new_sparse();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Float64Type>("B", 3.2).unwrap();
         builder.append::<Int32Type>("A", 34).unwrap();
@@ -1658,7 +1547,7 @@ mod tests {
 
     #[test]
     fn test_filter_union_array_sparse_with_nulls() {
-        let mut builder = UnionBuilder::new_sparse(4);
+        let mut builder = UnionBuilder::new_sparse();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append::<Float64Type>("B", 3.2).unwrap();
         builder.append_null::<Float64Type>("B").unwrap();
@@ -1669,7 +1558,7 @@ mod tests {
         let c = filter(&array, &filter_array).unwrap();
         let filtered = c.as_any().downcast_ref::<UnionArray>().unwrap();
 
-        let mut builder = UnionBuilder::new_sparse(2);
+        let mut builder = UnionBuilder::new_sparse();
         builder.append::<Int32Type>("A", 1).unwrap();
         builder.append_null::<Float64Type>("B").unwrap();
         let expected_array = builder.build().unwrap();
