@@ -248,11 +248,10 @@
 #![deny(clippy::redundant_clone)]
 #![warn(missing_debug_implementations)]
 
-pub mod alloc;
+pub use arrow_buffer::{alloc, buffer};
+
 pub mod array;
 pub mod bitmap;
-pub mod buffer;
-mod bytes;
 pub mod compute;
 #[cfg(feature = "csv")]
 pub mod csv;
@@ -269,6 +268,7 @@ pub mod json;
 #[cfg(feature = "pyarrow")]
 pub mod pyarrow;
 pub mod record_batch;
+pub mod row;
 pub mod temporal_conversions;
 pub mod tensor;
 pub mod util;
