@@ -15,12 +15,12 @@
 // specific language governing permissions and limitations
 // under the License.
 
+pub use arrow_buffer::{bit_chunk_iterator, bit_util};
+
 #[cfg(feature = "test_utils")]
 pub mod bench_util;
-pub mod bit_chunk_iterator;
 pub mod bit_iterator;
 pub(crate) mod bit_mask;
-pub mod bit_util;
 #[cfg(feature = "test_utils")]
 pub mod data_gen;
 pub mod display;
